package weather

import (
	"context"

	"gitea.libretechconsulting.com/rmcguire/go-app/pkg/config"
	"gitea.libretechconsulting.com/rmcguire/go-app/pkg/otel"
	"github.com/rs/zerolog"
	"go.opentelemetry.io/otel/metric"
)

func MustInitMetrics(appCtx context.Context) *WeatherMetrics {
	wm := &WeatherMetrics{
		appCtx:   appCtx,
		cfg:      config.MustFromCtx(appCtx),
		recorder: &MetricRecorder{ctx: appCtx, l: zerolog.Ctx(appCtx)},
	}

	wm.meter = otel.GetMeter(appCtx, "weather", "metrics")

	// Weather Metrics
	wm.TempOutdoorF, _ = wm.meter.Float64Gauge(MetricPrefix+"_temp_outdoor_f",
		metric.WithDescription("Outdoor Temperature in Faherenheit"))
	wm.TempIndoorF, _ = wm.meter.Float64Gauge(MetricPrefix+"_temp_indoor_f",
		metric.WithDescription("Indoor Temperature in Faherenheit"))
	wm.HumidityOudoor, _ = wm.meter.Int64Gauge(MetricPrefix+"_humidity_outdoor",
		metric.WithDescription("Outdoor Humidity %"))
	wm.HumidityIndoor, _ = wm.meter.Int64Gauge(MetricPrefix+"_humidity_indoor",
		metric.WithDescription("Indoor Humidity %"))
	wm.WindSpeedMPH, _ = wm.meter.Float64Gauge(MetricPrefix+"_wind_speed_mph",
		metric.WithDescription("Wind Speed in MPH"))
	wm.WindGustMPH, _ = wm.meter.Float64Gauge(MetricPrefix+"_wind_gust_mph",
		metric.WithDescription("Wind Gust in MPH"))
	wm.MaxDailyGust, _ = wm.meter.Float64Gauge(MetricPrefix+"_max_daily_gust",
		metric.WithDescription("Max Daily Wind Gust"))
	wm.WindDir, _ = wm.meter.Int64Gauge(MetricPrefix+"_wind_dir",
		metric.WithDescription("Wind Direction in Degrees"))
	wm.WindDirAvg10m, _ = wm.meter.Int64Gauge(MetricPrefix+"_wind_dir_avg_10m",
		metric.WithDescription("Wind Direction 10m Average"))
	wm.UV, _ = wm.meter.Int64Gauge(MetricPrefix+"_uv",
		metric.WithDescription("UV Index"))
	wm.SolarRadiation, _ = wm.meter.Float64Gauge(MetricPrefix+"_solar_radiation",
		metric.WithDescription("Solar Radiation in W/㎡"))
	wm.HourlyRainIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_hourly_rain_in",
		metric.WithDescription("Hourly Rain in Inches"))
	wm.EventRainIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_event_rain_in",
		metric.WithDescription("Event Rain in Inches"))
	wm.DailyRainIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_daily_rain_in",
		metric.WithDescription("Daily Rain in Inches"))
	wm.WeeklyRainIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_weekly_rain_in",
		metric.WithDescription("Weekly Rain in Inches"))
	wm.MonthlyRainIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_monthly_rain_in",
		metric.WithDescription("Monthly Rain in Inches"))
	wm.YearlyRainIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_yearly_rain_in",
		metric.WithDescription("Yearly Rain in Inches"))
	wm.TotalRainIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_total_rain_in",
		metric.WithDescription("Total Rain in Inches"))
	wm.BatteryStatus, _ = wm.meter.Int64Gauge(MetricPrefix+"_battery_status",
		metric.WithDescription("Per-component battery status"))
	wm.BaromRelativeIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_barometric_pressure_relative_in",
		metric.WithDescription("Relative Pressure in Inches of Mercury"))
	wm.BaromAbsoluteIn, _ = wm.meter.Float64Gauge(MetricPrefix+"_barometric_pressure_absolute_in",
		metric.WithDescription("Absolute Pressure in Inches of Mercury"))
	wm.DewPointF, _ = wm.meter.Float64Gauge(MetricPrefix+"_dew_point_f",
		metric.WithDescription("Dew Point in Faherenheit"))
	wm.WindChillF, _ = wm.meter.Float64Gauge(MetricPrefix+"_wind_chill_f",
		metric.WithDescription("Wind Chill in Faherenheit"))

	// Lightning Sensor
	wm.LightningCountDay, _ = wm.meter.Int64Gauge(MetricPrefix+"_lightning_day",
		metric.WithDescription("Count of lightning strikes for current day"))
	wm.LightningLastDistance, _ = wm.meter.Int64Gauge(MetricPrefix+"_lightning_last_distance",
		metric.WithDescription("Last measured lightning distance"),
		metric.WithUnit("km"))
	wm.LightningLastTime, _ = wm.meter.Int64Gauge(MetricPrefix+"_lightning_last_time",
		metric.WithDescription("Last lightning time (epoch)"),
		metric.WithUnit("s"))

	// Temp and Humidity Sensors
	wm.SensorTempF, _ = wm.meter.Float64Gauge(MetricPrefix+"_sensor_temp_f",
		metric.WithDescription("Temperature Sensor in Faherenheit"))
	wm.SensorHumidity, _ = wm.meter.Int64Gauge(MetricPrefix+"_sensor_humidity",
		metric.WithDescription("Humidity % Sensor"))

	// Internal Telemetry
	wm.UpdatesReceived, _ = wm.meter.Int64Counter(MetricPrefix+"_updates_received",
		metric.WithDescription("Metric Updates Processed by Exporter"))

	return wm
}
