package redis

import (
	"context"
	"encoding/json"
	"errors"
	"slices"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"
	"go.opentelemetry.io/otel/trace"

	pb "gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/api/v1alpha1/weather"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/util"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (r *RedisRecorder) Get(ctx context.Context, req *pb.GetWeatherRequest) (
	[]*weather.WeatherUpdate, error,
) {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.get", trace.WithAttributes(
		attribute.Int("limit", util.GetLimitFromReq(req)),
	))
	defer span.End()

	return r.get(ctx, req)
}

func (r *RedisRecorder) get(ctx context.Context, req *pb.GetWeatherRequest) (
	[]*weather.WeatherUpdate, error,
) {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.get.redis")
	defer span.End()

	limit := util.GetLimitFromReq(req)
	if limit < 1 {
		limit = r.keep
	}

	span.SetAttributes(attribute.Int("limit", limit))
	r.log.Debug().Int("limit", limit).Msg("getting updates from redis")

	datas, err := r.redis.LRange(ctx, r.Key(), 0, int64(limit)-1).Result()
	if err != nil {
		span.RecordError(err)
		span.SetStatus(codes.Error, err.Error())
		r.log.Err(err).Send()
		return nil, err
	}

	span.AddEvent("redis queried")
	r.log.Debug().Int("results", len(datas)).Msg("redis queried")

	updates, err := jsonDatasToUpdates(datas)
	if err != nil {
		span.RecordError(err)
		span.SetStatus(codes.Error, err.Error())
		r.log.Err(err).Send()
	} else {
		span.SetStatus(codes.Ok, "")
	}

	span.AddEvent("results unmarshalled")
	span.SetAttributes(attribute.Int("results", len(updates)))

	filtered := util.ApplyOptsToUpdates(updates, limit, req.Opts)

	span.AddEvent("results filtered")
	span.SetAttributes(
		attribute.Int("filteredResults", len(filtered)),
		attribute.Int("resultsFiltered", len(updates)-len(filtered)),
	)

	r.log.Debug().
		Int("updatesRetrieved", len(updates)).
		Int("updatesAfterFiltering", len(filtered)).
		Int("updatesFiltered", len(updates)-len(filtered)).
		Msg("updates retrieved from redis")

	return filtered, err
}

func jsonDatasToUpdates(datas []string) ([]*weather.WeatherUpdate, error) {
	var errs error
	updates := make([]*weather.WeatherUpdate, 0, len(datas))

	for _, data := range datas {
		update := new(weather.WeatherUpdate)
		err := json.Unmarshal([]byte(data), update)
		errs = errors.Join(errs, err)

		if err == nil {
			updates = append(updates, update)
		}
	}

	return slices.Clip(updates), errs
}
