package redis

import (
	"context"
	"encoding/json"

	"github.com/redis/go-redis/v9"
	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"
	"go.opentelemetry.io/otel/trace"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/util"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (r *RedisRecorder) Set(ctx context.Context, u *weather.WeatherUpdate) error {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.set", trace.WithAttributes(
		attribute.String("stationName", u.GetStationName()),
		attribute.String("stationType", util.DerefStr(u.StationType)),
	))
	defer span.End()

	r.Lock()
	defer r.Unlock()

	// First ensure we can prepare our payload
	data, err := json.Marshal(u)
	if err != nil {
		span.RecordError(err)
		span.SetStatus(codes.Error, err.Error())

		r.log.Err(err).Send()
		return err
	}

	return r.set(ctx, data)
}

func (r *RedisRecorder) set(ctx context.Context, data []byte) error {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.set.push", trace.WithAttributes(
		attribute.Int("updateBytes", len(data)),
	))
	defer span.End()

	// Atomic, push and trim
	var count *redis.IntCmd
	rErr, err := r.redis.TxPipelined(ctx, func(pipe redis.Pipeliner) error {
		pipe.LPush(ctx, r.Key(), data)
		pipe.LTrim(ctx, r.Key(), 0, int64(r.keep)-1)
		count = pipe.LLen(ctx, r.Key())
		pipe.Expire(ctx, r.Key(), r.keyTTL)
		return nil
	})
	if err != nil {
		for _, cmd := range rErr {
			span.RecordError(cmd.Err())
		}
		span.SetStatus(codes.Error, err.Error())
		r.log.Err(err).Send()
		return err
	}

	// Get new update count

	r.log.Debug().
		Int("updateBytes", len(data)).
		Int64("updateCount", count.Val()).
		Str("redis", r.redis.String()).
		Str("key", r.Key()).
		Msg("pushed update to redis")

	span.SetAttributes(attribute.Int64("updateCount", count.Val()))
	span.SetStatus(codes.Ok, "")
	return nil
}
